from PIL import Image
from viz_image_deobfuscate import deobfuscate_image


def main() -> None:
    import argparse

    parser = argparse.ArgumentParser(description="Deobfuscate manage page image.")
    parser.add_argument("obfuscated_image", help="Path to the obfuscated image.")
    parser.add_argument(
        "deobfuscated_image", help="Output path to the obfuscated image."
    )

    args = parser.parse_args()

    deobfuscated_image: Image = deobfuscate_image(args.obfuscated_image)
    if deobfuscate_image:
        deobfuscated_image.save(args.deobfuscated_image)
        print(f"Successfully deobfuscated image at: {args.deobfuscated_image}")
    else:
        print(f"Unable to deobfuscate image, check image Exif data.")
        exit(1)


if __name__ == "__main__":
    main()


from dataclasses import dataclass
from typing import Any, Dict, List, Tuple
from math import floor
from PIL import Image
import piexif
import logging


@dataclass
class Point:
    x: int
    y: int

    def loc(self) -> Tuple[int, int]:
        return (self.x, self.y)


@dataclass
class Size:
    width: int
    height: int


@dataclass
class Box:
    point: Point
    size: Size

    def region(self) -> Tuple[int, int, int, int]:
        return (
            self.point.x,
            self.point.y,
            self.point.x + self.size.width,
            self.point.y + self.size.height,
        )


def _get_exif_key(image_loc: str) -> List[int]:
    metadata: Dict[str, Any] = piexif.load(image_loc)
    ifd: Dict[int, Any] = metadata.get("Exif")
    if ifd:
        tag: int = 42016  # 0xA420 # ImageUniqueID
        key: bytes = ifd.get(tag)
        if key:
            hex: List[str] = key.decode().split(":")
            return [int(h, 16) for h in hex]
        else:
            logging.warning(
                f"Unable to find Exif ImageUniqueID ({tag}) ifd tag for image: {image_loc}"
            )
    else:
        logging.warning(f"Unable to find Exif metadata for image: {image_loc}")
    return []


def _draw_image(
    dest_image: Image, src_image: Image, src_box: Box, dest_point: Point
) -> None:
    cropped_image: Image = src_image.crop(box=src_box.region())
    dest_image.paste(cropped_image, dest_point.loc())


def deobfuscate_image(image_name: str) -> Image:
    keys: List[int] = _get_exif_key(image_name)
    if not keys:
        return None

    obfuscated_image: Image = Image.open(image_name)

    spacing: int = 10
    columns: int = 10
    rows: int = 15
    width: int = obfuscated_image.width - (columns - 1) * spacing
    height: int = obfuscated_image.height - (rows - 1) * spacing

    deobfuscated_image: Image = Image.new("RGB", size=(width, height), color="white")

    tile_width: int = floor(width / 10)
    tile_height: int = floor(height / 15)

    # The bounding 'tiles' are the actual edges of the page, so copy the over.
    _draw_image(  # top
        deobfuscated_image,
        obfuscated_image,
        Box(Point(0, 0), Size(width, tile_height)),
        Point(0, 0),
    )
    _draw_image(  # left
        deobfuscated_image,
        obfuscated_image,
        Box(
            Point(0, tile_height + spacing), Size(tile_width, height - 2 * tile_height)
        ),
        Point(0, tile_height),
    )
    _draw_image(  # bottom
        deobfuscated_image,
        obfuscated_image,
        Box(
            Point(0, (rows - 1) * (tile_height + spacing)),
            Size(width, obfuscated_image.height - (rows - 1) * (tile_height + spacing)),
        ),
        Point(0, (rows - 1) * tile_height),
    )
    _draw_image(  # right
        deobfuscated_image,
        obfuscated_image,
        Box(
            Point((columns - 1) * (tile_width + spacing), tile_height + spacing),
            Size(tile_width + (width - columns * tile_width), height - 2 * tile_height),
        ),
        Point((columns - 1) * tile_width, tile_height),
    )

    for idx, key in enumerate(keys):
        # move each center tile to their proper location
        _draw_image(
            deobfuscated_image,
            obfuscated_image,
            Box(
                Point(
                    floor((idx % 8 + 1) * (tile_width + spacing)),
                    floor((floor(idx / 8) + 1) * (tile_height + spacing)),
                ),
                Size(floor(tile_width), floor(tile_height)),
            ),
            Point(
                floor((key % 8 + 1) * tile_width),
                floor((floor(key / 8) + 1) * tile_height),
            ),
        )
    return deobfuscated_image




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['viz_image_deobfuscate']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.1.0,<10.0.0', 'piexif>=1.1.3,<2.0.0']

entry_points = \
{'console_scripts': ['image-deobfuscate-cli = viz_image_deobfuscate.cli:main']}

setup_kwargs = {
    'name': 'viz-image-deobfuscate',
    'version': '0.1.4',
    'description': 'Deobfuscate Viz-style manga images.',
    'long_description': '# Viz Manga Deobfuscator\n\nViz manga pages are delivered to the browser as obfuscated images and the client JS is responsible for deobfuscation of those images for the viewer to read. This program reproduces the deobfuscate logic to produce a readable image.\n\n`obfuscated image:`\n![obfuscated image](https://raw.githubusercontent.com/minormending/viz-image-deobfuscate/main/images/raw1.jpg)\n\n`deobfuscated image:`\n![deobfuscated image](https://raw.githubusercontent.com/minormending/viz-image-deobfuscate/main/images/page1.jpg)\n\nThe image Exif metadata stores a hex digest to deobfuscate the image. Using the each byte value of the digest with it\'s position in the digest, we can select the appropriate tile in the obfuscated image and put it in the proper place in the deobfuscated image.  \n\nDISCLAIMER: I am not licensed or affiliated with Viz Media and this repository is meant for informational purposes only.\n\n# Installation \n```\npip install viz-image-deobfuscate \n```\n\n# Usage\nThis package exposes `deobfuscate_image` that accepts a path to an image and returns an PIL `Image` of the deobfuscated image.\n\n```\nfrom viz_image_deobfuscate import deobfuscate_image\n\ndeobfuscated = deobfuscate_image("raw.jpg")\ndeobfuscated.save("page.jpg")\n```\n\n# CLI Usage\nBundled with this package is a CLI tool for scripting/testing purposes.\n\n```\nusage: image-deobfuscate-cli [-h] obfuscated_image deobfuscated_image\n\nDeobfuscate manage page image.\n\npositional arguments:\n  obfuscated_image    Path to the obfuscated image.\n  deobfuscated_image  Output path to the obfuscated image.\n\noptions:\n  -h, --help          show this help message and exit\n```\n\n## Example\n```\n>>> image-deobfuscate-cli raw1.jpg page1.jpg\n\nSuccessfully deobfuscated image at: page1.jpg\n```\n\n# Docker\nAlternatively, you can build your own docker container to run the CLI or download an already built container from [Docker Hub](https://hub.docker.com/r/minormending/viz-image-deobfuscate)\n\n```\n>>> docker build -t viz .\n>>> docker run -v /home/user/images/:/app/images viz images/raw1.jpg images/page1.jpg\n\nSuccessfully deobfuscated image at: images/page1.jpg\n```',
    'author': 'Kevin Ramdath',
    'author_email': 'krpent@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/minormending/viz-image-deobfuscate',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
